#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0


cost = "service.main.cost"
service_license = "service.attributes.main.license"
sample = "service.attributes.additionalInformation.links.type"
categories = "service.attributes.additionalInformation.categories"
tags = "service.attributes.additionalInformation.tags"
created = "created"
dataToken = "dataToken"
datePublished = "service.attributes.main.datePublished"
dateCreated = "service.attributes.main.dateCreated"
updated_frequency = "service.attributes.additionalInformation.updateFrequency"
metadata_type = "service.attributes.main.type"
service_type = "service.type"

list_indexes = [cost, service_license, sample, categories, tags,
                created, dataToken, updated_frequency, service_type, metadata_type]
